import React, { Component } from 'react';
import PropTypes from 'prop-types';
import Range from "./Range";
import Node from "./nodes/Node";
import { debounce, getHTMLtag } from "./helpers";
export default class HTMLhighlight extends Component {
  constructor(props) {
    super(props);
    this.state = {
      text: ''
    };
    this.dismissMouseUp = 0;
  }

  shouldComponentUpdate(newProps) {
    return newProps.ranges.length !== this.props.ranges.length || newProps.HTMLinput !== this.props.HTMLinput || newProps.enabled !== this.props.enabled;
  }

  getRange(charIndex) {
    return this.props.ranges && this.props.ranges.find(range => charIndex >= range.start && charIndex <= range.end);
  }

  onMouseOverHighlightedWord(range, visible) {
    if (visible && this.props.onMouseOverHighlightedWord) {
      this.props.onMouseOverHighlightedWord(range);
    }
  }

  getLetterNode(charIndex, range) {
    return React.createElement(Node, {
      id: this.props.id,
      range: range,
      charIndex: charIndex,
      key: `${this.props.id}-${charIndex}`,
      highlightStyle: this.props.highlightStyle
    }, this.props.text[charIndex]);
  }

  mouseEvent() {
    if (!this.props.enabled) {
      return false;
    }

    let text = '';

    if (window.getSelection) {
      text = window.getSelection().toString();
    } else if (document.selection && document.selection.type !== 'Control') {
      text = document.selection.createRange().text;
    }

    if (!text || !text.length) {
      return false;
    }

    const range = window.getSelection().getRangeAt(0);
    const startContainerPosition = parseInt(range.startContainer.parentNode.dataset.position);
    const endContainerPosition = parseInt(range.endContainer.parentNode.dataset.position);
    const startHL = startContainerPosition < endContainerPosition ? startContainerPosition : endContainerPosition;
    const endHL = startContainerPosition < endContainerPosition ? endContainerPosition : startContainerPosition;
    const rangeObj = new Range(startHL, endHL, text, Object.assign({}, this.props, {
      ranges: undefined
    }));
    this.props.onTextHighlighted(rangeObj);
  }

  onMouseUp(event) {
    debounce(() => {
      if (this.doucleckicked) {
        this.doucleckicked = false;
        this.dismissMouseUp++;
      } else if (this.dismissMouseUp > 0) {
        this.dismissMouseUp--;
      } else {
        this.mouseEvent.bind(this)();
      }
    }, 200).bind(this)();
  }

  rangeRenderer(letterGroup, range, textCharIndex, onMouseOverHighlightedWord) {
    return this.props.rangeRenderer ? this.props.rangeRenderer(letterGroup, range, textCharIndex, onMouseOverHighlightedWord) : letterGroup;
  }

  getRanges() {
    const newText = [];
    let lastRange;
    let viewCharIndex = 0; // For all the characters on the text

    for (let textCharIndex = 0; textCharIndex < this.props.HTMLinput.length; textCharIndex++) {
      const range = this.getRange(textCharIndex); // Get the current character node

      const HTMLtag = getHTMLtag(textCharIndex, this.props.HTMLinput);
      const node = this.getLetterNode(viewCharIndex++, range);

      if (HTMLtag.length) {
        newText.push(HTMLtag);
        textCharIndex += HTMLtag.length - 1;
      }

      if (!range) {
        newText.push(node);
        continue;
      } // If the char is in range


      lastRange = range; // We put the first range node on the array

      const letterGroup = [node]; // For all the characters in the highlighted range

      let rangeCharIndex = viewCharIndex + 1;

      for (; rangeCharIndex < parseInt(range.end) + 1; rangeCharIndex++) {
        letterGroup.push(this.getLetterNode(rangeCharIndex, range));
        viewCharIndex = rangeCharIndex;
      }

      newText.push(this.rangeRenderer(letterGroup, range, viewCharIndex, this.onMouseOverHighlightedWord.bind(this)));
    }

    if (lastRange) {
      // Callback function
      this.onMouseOverHighlightedWord(lastRange, true);
    }

    return newText;
  }

  render() {
    const newText = this.getRanges();
    return React.createElement("div", {
      style: this.props.style,
      onMouseUp: this.onMouseUp.bind(this),
      onDoubleClick: this.onDoubleClick.bind(this),
      dangerouslySetInnerHTML: newText
    });
  }

}
SeraphinAnno.propTypes = {
  ranges: PropTypes.array,
  id: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  HTMLinput: PropTypes.string,
  enabled: PropTypes.bool,
  onMouseOverHighlightedWord: PropTypes.func,
  onTextHighlighted: PropTypes.func,
  style: PropTypes.object,
  rangeRenderer: PropTypes.func
};